from unittest import TestCase
from k9.helper import *
import pprint

from k9.pretty_object import PrettyObject

po = PrettyObject()

pp = pprint.PrettyPrinter(indent=2, width=120)
class TestCron(TestCase):

    def test_create_namespace(self):
        set_default_namespace("unit-test")
        
        create_namespace()
        self.assertTrue(namespace_exists())

        delete_namespace()
        self.assertFalse(namespace_exists())

    def test_yaml(self):
        view_yaml('craft-deploy-dev.yml')




class PrettyObject:

    # Establishes size that list structures would extend
    # to line by line display
    dictLimit  = 1
    listLimit  = 1
    setLimit   = 1
    tupleLimit = 1

    INDENT = "    "

    def pretty_dict(self, obj: dict, indent=""):
        if len(obj) < self.dictLimit:
            return self.pretty_dict_line(obj, indent)

        result = "{\n"
        for key in obj:
            result += indent + self.INDENT + key + ": ";
            result += self.string(obj[key], indent + self.INDENT) + "\n"

        result += indent + "}"
        return result

    def pretty_dict_line(self, obj: dict, indent=""):
        if len(obj) == 0:
            return "{}"

        first = True
        result = "{"
        for key in obj:
            if not first:
                result += ", "
            first = False
            result += key + ": ";
            result += self.string(obj[key], indent)

        result += "}"
        return result

    def pretty_list(self, obj, indent=""):
        if len(obj) < self.listLimit:
            return self.pretty_list_line(obj, indent)

        result = "[\n"
        for item in obj:
            result += indent + self.INDENT + self.string(item, indent + self.INDENT) + "\n"

        result += indent + "]"
        return result

    def pretty_list_line(self, obj, indent=""):
        if len(obj) == 0:
            return "[]"

        first = True;
        result = "["
        for item in obj:
            if not first:
                result += ", "
            first = False
            result += self.string(item, indent)

        result += "]"

        return result

    def pretty_tuple(self, obj, indent=""):
        if len(obj) < self.listLimit:
            return self.pretty_tuple_line(obj, indent)

        result = "(\n"
        for item in obj:
            result += indent + self.INDENT + self.string(item, indent + self.INDENT) + "\n"

        result += indent + ")"
        return result

    def pretty_tuple_line(self, obj, indent=""):
        if len(obj) == 0:
            return "()"

        first = True;
        result = "("
        for item in obj:
            if not first:
                result += ", "
            first = False
            result += self.string(item, indent)

        result += ")"

        return result

    def string(self, obj, indent=""):
        if isinstance(obj, dict):
            return self.pretty_dict(obj, indent)

        if isinstance(obj, list):
            return self.pretty_list(obj, indent)

        if isinstance(obj, tuple):
            return self.pretty_tuple(obj, indent)

        if isinstance(obj, str):
            return '"' + obj + '"'

        else:
            return str(obj)

    def print(self, obj):
        print(self.string(obj))

    def print_table(self, obj):
        if not isinstance(obj, list):
             print("Object is not a table:")
             self.print(obj)
             return

        # Capture stats on the table
        lengths = {}
        order = []
        for record in obj:
            for key in record:
                value  = str(record[key])
                length = len(value)

                if not key in lengths:
                    keylen = len(key)
                    if keylen > length:
                        length = keylen

                    lengths.update({key: length})
                    order.append(key)
                    continue

                if lengths[key] < length:
                    lengths.update({key: length})

        # Print the header
        line = ""
        for key in order:
            line += key.ljust(lengths[key])
            line += "  "
        print(line)

         # Print underline
        line = ""
        for key in order:
            line += "".ljust(lengths[key], '-')
            line += "  "
        print(line)

        # Print table content
        for record in obj:
            line = ""
            for key in order:
                if key in record:
                    value = str(record[key])
                else:
                    value = ""
                line += value.ljust(lengths[key]+2)

            print(line)


def get_tags(tag):
    result = {}
    for vpair in tag:
        result.update({vpair.get('Key'): vpair.get('Value')})

    return result


def get_tag(tags, key):
    for vpair in tags:
        if vpair['Key'] == key:
            return vpair['Value']
    return ""

def get_name(tags):
    if tags is None:
        return ''
    for vpair in tags:
        if vpair['Key'] == 'Name':
            return vpair['Value']
    return ''

def get_true(value):
    '''For easy viewing, only display True'''

    if value:
        return 'True'
    else:
        return ''




from kubernetes import client, config
from k9.pretty_object import PrettyObject
import yaml

po = PrettyObject()
config.load_kube_config()

v1 = client.CoreV1Api()
appsv1 = client.AppsV1Api()
rbac = client.RbacAuthorizationV1Api()
api = client.ApiextensionsV1beta1Api()

default_namespace = None


###########################################################################
# Utility Functions
###########################################################################

def last_word(value: str):
    """ Splits out the word after the last slash in a string.  K8
        objects are expressed in a path style name                 """
    return value.split('/')[-1:][0]


def view_yaml(fn: str):
    """ Dumps out yaml file in simplified JSON format for easy viewing. This
        is useful when constructing the body of a request that matches aknown yaml format."""
    file = None
    try:
        file = open(fn)
        po.print(yaml.safe_load(file))

    finally:
        if file is not None:
            file.close()


###########################################################################
# Namespaces
###########################################################################

def set_default_namespace(namespace: str = None):
    """sets the default namespace for all functions that need namespace."""

    global default_namespace
    default_namespace = namespace
    print(f"default_namespace={default_namespace}")


def get_default_namespace():
    global default_namespace
    return default_namespace


def list_namespaces():
    """Returns a list of dictionaries with name and status."""

    return [
        {'name': last_word(namespace.metadata.self_link), 'status': namespace.status.phase}
        for namespace in v1.list_namespace().items
    ]


def namespace_exists(namespace: str = None):
    if namespace is None:
        namespace = get_default_namespace()

    found = [
        ns['name']
        for ns in list_namespaces()
        if ns['status'] == 'Active' and ns['name'] == namespace
    ]

    return len(found) > 0


def create_namespace(namespace: str = None):
    if namespace is None:
        namespace = get_default_namespace()

    body = \
        {
            "apiVersion": "v1",
            "kind": "Namespace",
            "metadata": {
                "name": namespace,
            }
        }

    return v1.create_namespace(body)


def delete_namespace(namespace: str = None):
    if namespace is None:
        namespace = get_default_namespace()

    return v1.delete_namespace(namespace)


###########################################################################
# Pods
###########################################################################

def list_pods(namespace: str = None):
    if namespace is None:
        namespace = get_default_namespace()

    pod_list = v1.list_namespaced_pod(namespace)
    return [
        {'name': pod.metadata.name, 'status': pod.status.phase, 'ip': pod.status.pod_ip, 'labels': pod.metadata.labels, 'reason': pod.status.reason, 'start_time': pod.status.start_time}
        for pod in pod_list.items
    ]


###########################################################################
# Deployments
###########################################################################

def list_deployments(namespace: str = None):
    if namespace is None:
        namespace = get_default_namespace()

    return [
        last_word(deployment.metadata.self_link)
        for deployment in appsv1.list_namespaced_deployment(namespace).items
    ]


def get_deployment(name: str, namespace: str = None):
    if namespace is None:
        namespace = get_default_namespace()

    return appsv1.read_namespaced_deployment(name, namespace)


###########################################################################
# Service Accounts
###########################################################################

def create_service_account(name: str, namespace: str = None):
    if namespace is None:
        namespace = get_default_namespace()

    body =\
        { 'apiVersion': 'v1',
          'kind': 'ServiceAccount',
          'metadata' : { 'name': name }
        }

    return v1.create_namespaced_service_account(namespace, body)


def delete_service_account(name: str, namespace: str = None):
    if namespace is None:
        namespace = get_default_namespace()

    return v1.delete_namespaced_service_account(name, namespace)


def list_service_account(name: str, namespace: str = None):
    if namespace is None:
        namespace = get_default_namespace()

    return v1.read_namespaced_service_account(name, namespace)


###########################################################################
# Cluster Roles
###########################################################################

def list_cluster_roles():
    return [
        {'name': last_word(role.metadata.self_link).replace('%3A', ':'), 'created': role.metadata.creation_timestamp}
        for role in rbac.list_cluster_role().items
    ]


def create_cluster_role(body: dict):
    return rbac.create_cluster_role(body)


def delete_cluster_role(name: str):
    return rbac.delete_cluster_role(name)


def get_cluster_role(name: str):
    try:
        return rbac.read_cluster_role(name)
    except Exception as e:
        return None


def create_cluster_role_binding(binding: str, role: str, sa: str, namespace: str = None):
    if namespace is None:
        namespace = get_default_namespace()

    body =\
        {
            'apiVersion': 'rbac.authorization.k8s.io/v1',
            'kind': 'ClusterRoleBinding',
            'metadata': {'name': binding},
            'roleRef': {
                'apiGroup': '',
                'kind': 'ClusterRole',
                'name': role
            },
            'subjects': {
                'kind': 'ServiceAccount',
                'name': sa,
                'namespace': namespace
            }
        }

    return rbac.create_cluster_role_binding(body)


def delete_cluster_role_binding(binding: str):
    pass


def get_cluster_role_binding(binding: str):
    pass

###########################################################################
# Custom Functions
###########################################################################

def create_ecr_login_role(name: str):
    # Create service account AWS login role
    body = \
    { 'apiVersion': 'rbac.authorization.k8s.io/v1',
      'kind': 'ClusterRole',
      'metadata': { 'name': name },
      'rules': [
        { 'apiGroups': [''],
          'resources': ['serviceaccounts'],
          'verbs': ['get', 'patch'] },

        { 'apiGroups': [''],
          'resources': ['secrets'],
          'verbs': ['create', 'delete']}
               ]
    }

    return create_cluster_role(body)

cluster_role = 'test-ecr-role'
#pprint(create_ecr_login_role(cluster_role))
#pprint(get_cluster_role(cluster_role))
#pprint(delete_cluster_role(cluster_role))






import pathlib
from setuptools import setup, find_packages
import toml

def get_install_requirements():
    try:
        # read my pipfile
        with open ('Pipfile', 'r') as fh:
            pipfile = fh.read()
        # parse the toml
        pipfile_toml = toml.loads(pipfile)
        
    except FileNotFoundError:
        return []

    try:
        required_packages = pipfile_toml['packages'].items()
        # if the package's key isn't there then just return an empty
        # list
        return ["{0}{1}".format(pkg,ver) 
                if ver != "*" else pkg for pkg,ver in required_packages]
                
    except KeyError:
        return []
        
     # If a version/range is specified in the Pipfile honor it
     # otherwise just list the package


# The directory containing this file
HERE = pathlib.Path(__file__).parent

# The text of the README file
README = (HERE / "README.md").read_text()

# This call to setup() does all the work
setup(
    name="k9",
    version="0.0.6",
    description="Automation Helper for K8 (Kubernetes)",
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://gitlab.com/python-cicd/k9",
    author="SimonComputing, Inc.",
    author_email="simon@simoncomputing.com",
    license="MIT",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
    ],
    packages=find_packages(exclude=("test", "venv", "build", "dist")),
    include_package_data=True,
    install_requires=get_install_requirements(),
    entry_points={
        "console_scripts": [
            "k9=k9.__main__:main",
        ]
    },
)
